/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Harmonics.hpp
 *
 *  Created on: 2010/12/09
 *      Author: HARK Development Team
 */

#ifndef HARMONICS_HPP_
#define HARMONICS_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <HarmonicSound.hpp>

namespace NSHarkBinaural
{

class Harmonics
{
public:
    typedef boost::shared_ptr<HarmonicSound> HarmonicSoundPtr;
    typedef std::vector<HarmonicSoundPtr> HarmonicSoundVector;
    typedef boost::shared_ptr<HarmonicSoundVector> HarmonicSoundVectorPtr;
    typedef std::vector<Band> BandVector;
    typedef boost::shared_ptr<BandVector> BandVectorPtr;

    /**
     * 与えられた代表値リストから、調波構造（ハーモニックス）を抽出する.
     * @param proxyBands 代表値の周波数バンドのリスト
     * @return 抽出された調波構造一覧
     */
    static HarmonicSoundVectorPtr
    ExtractAllHarmonics(const BandVectorPtr proxyBands);

    /**
     * 与えられた調波構造一覧の中で、人間の声の調波構造を取得する.
     * ヒューリスティックな方法を用いて、与えられた調波構造の中から
     * 人間の声と思われる調波構造を発見する。
     * @param harmonicsList 調波構造一覧
     * @param bandSize 周波数バンドの種類数(ndft / 2 + 1)
     * @param bi 周波数バンド情報
     * @param threshold CheckHarmonics()の戻り値がこの値以上なら人間の声と判断する
     * @return 人間の声の調波構造一覧
     */
    static HarmonicSoundVectorPtr
    GetHumanHarmonics(const HarmonicSoundVectorPtr harmonicsList, size_t bandSize, const BandInformation& bi, int threshold = 2);

//private:
public:
    /**
     * 与えられた基音のN倍音を取得する.
     * @param proxyBands 代表値の周波数バンドのリスト
     * @param fundamental 基音（proxyBandsでのインデックス）
     * @param n 取得したい倍音の、基音に対する倍率（X倍音ならXを指定する）
     * @return 倍音がある場合はその倍音（代表値の周波数バンド）。ない場合はBand()
     */
    static Band
    GetNHarmonic(const BandVectorPtr proxyBands, int fundamental, int n);

    /**
     * 与えられた代表値リストから、与えられた音程を基音とする調波構造を抽出する.
     * @param proxyBands 代表値の周波数バンドのリスト
     * @param fundamental 基音（proxyBandsでのインデックス）
     * @return 調波構造がある場合はその調波構造。ない場合はnull
     */
    static HarmonicSoundPtr
    GetHarmonics(const BandVectorPtr proxyBands, int fundamental);

    /**
     * check harmonics correctness.
     * I can not help but using heuristic rules, I suppose.
     *
     * @param sound 調波構造
     * @param bi 周波数バンド情報
     * @return -4 -> false
     *         -3 -> false no 1,2
     *         -2 -> false no 1,3
     *         -1 -> false no 2,4,6
     *          0 -> false fundamental > 300Hz
     *          1 -> true  pure tone
     *          2 -> true  nh-c <= 1
     *          3 -> true  nh<=5,c>=3
     *          4 -> true  nh>=5,c>=3
     */
    static int
    CheckHarmonics(const HarmonicSoundPtr sound, const BandInformation& bi);
};

}

#endif /* HARMONICS_HPP_ */
